/* Teknisa - Tecnologia em Informtica

Nome: 3938_CDPFUNS - ESTOQUE_SEMIACABADO

Descrio:
Rotina para levantar todos os semiacabados necessrios para a produo e 
verificar o estoque do dia disponvel gravando em uma tabela temporria.
Esta tabela ser lida e apagada em outras procedures.

Criao:
22/04/2016 - Srgio Pow

@..\Sql\End_Conv\SQL Server\3938_CDPFUNS.SQL
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'ESTOQUE_SEMIACABADO' AND
                 TYPE = 'P' 		       AND
	              U.UID = O.UID            AND
	              U.NAME = USER)
DROP PROCEDURE ESTOQUE_SEMIACABADO
GO

CREATE PROCEDURE ESTOQUE_SEMIACABADO
   @P_CDFILIAL      VARCHAR(4),
   @P_NRPEDIDO      VARCHAR(10),
   @P_CDPRODUTO     VARCHAR(10),
   @P_CDPROCFABRIC  VARCHAR(10),
   @P_QUANTIDADE    DOUBLE PRECISION
AS
   DECLARE @V_CDPRODESTO     VARCHAR(10);
   DECLARE @V_CDPRODEMANDA   VARCHAR(10);
   DECLARE @V_QTESTOQDIA     DOUBLE PRECISION;
   DECLARE @V_NOVAQT         DOUBLE PRECISION;
   DECLARE @V_QTDEMANDA      DOUBLE PRECISION;
   DECLARE @V_QTESTOQUE      DOUBLE PRECISION;
   
   DECLARE @V_SEMIACABADO_CDFILIAL       VARCHAR(4);
   DECLARE @V_SEMIACABADO_CDPRODUTO      VARCHAR(10);
   DECLARE @V_SEMIACABADO_CDPRODINSUMO   VARCHAR(10);
   DECLARE @V_SEMIACABADO_CDPROCFABRIC   VARCHAR(10);
   DECLARE @V_SEMIACABADO_QTAINSUMPREOP  DOUBLE PRECISION;

BEGIN
   SET @V_NOVAQT     = 0;
   SET @V_QTDEMANDA  = 0;
   SET @V_QTESTOQUE  = 0;
   SET @V_QTESTOQDIA = 0;
   
   -- Consulta os semiacabados do produto passado com parmetro.
   DECLARE C_SEMIACABADO CURSOR FOR

      SELECT I.CDFILIAL,
             I.CDPRODUTO,
             I.CDPRODINSUMO,
             F.CDPROCFABRIC,
             (TEKNISA.QTPREVI_MATPRIMA(I.CDFILIAL, I.CDPRODUTO, I.CDPROCFABRIC, I.CDETAPAPROD, I.CDPRODINSUMO, @P_QUANTIDADE, 'S')) AS QTAINSUMPREOP
        FROM PROCFABRICA F,
             INSUMETAPA  I
       WHERE (I.IDSEMIACETAP = 'S'            )
         AND (F.IDPROFABPADR = 'S'            )
         AND (F.CDFILIAL     = I.CDFILIAL     )
         AND (F.CDPRODUTO    = I.CDPRODINSUMO )
         AND (I.CDFILIAL     = @P_CDFILIAL    )
         AND (I.CDPRODUTO    = @P_CDPRODUTO   )
         AND (I.CDPROCFABRIC = @P_CDPROCFABRIC)
       ORDER BY I.CDPRODUTO, I.CDPRODINSUMO

      OPEN C_SEMIACABADO;
      FETCH NEXT FROM C_SEMIACABADO INTO @V_SEMIACABADO_CDFILIAL, @V_SEMIACABADO_CDPRODUTO, @V_SEMIACABADO_CDPRODINSUMO,
                                         @V_SEMIACABADO_CDPROCFABRIC, @V_SEMIACABADO_QTAINSUMPREOP;

      WHILE (@@FETCH_STATUS = 0) BEGIN
   
      IF @V_SEMIACABADO_QTAINSUMPREOP > 0 BEGIN

         -- Consulta o estoque do dia para o semiacabado.
         SELECT @V_CDPRODESTO = PE.CDPRODUTO,
                @V_QTESTOQDIA = SUM(QTESTDIACON)
           FROM POSESTCONS PE, PARAMPEDIDO PA, PRODUTO PR
          WHERE (PE.QTESTDIACON <>  0             )
            AND (PE.CDPRODUTO   = PR.CDPRODESTO   )
            AND (PE.CDFILIAL    = PA.CDFILIAL     )
            AND (ISNULL(PA.IDCONSESTGEROP, 'N') = 'S')
            AND (PE.CDFILIAL    = @V_SEMIACABADO_CDFILIAL    )
            AND (PR.CDPRODUTO   = @V_SEMIACABADO_CDPRODINSUMO)
            AND (PE.DTPOSESTCON = (
                                   SELECT MAX(PO.DTPOSESTCON)
                                     FROM POSESTCONS PO
                                    WHERE (PO.CDFILIAL     = PE.CDFILIAL )
                                      AND (PO.CDPRODUTO    = PE.CDPRODUTO)
                                      AND (PO.DTPOSESTCON <= (GETDATE()+1))
                                   )
                )
          GROUP BY PE.CDPRODUTO;

         IF @V_QTESTOQDIA > 0 BEGIN

            -- Verifica se a quantidade em estoque j est salva na tabela temporria.
            SELECT @V_CDPRODEMANDA = MAX(CDPRODUTO),
                   @V_QTDEMANDA    = MAX(QTDEMANDA),
                   @V_QTESTOQUE    = MAX(QTESTOQUE)
              FROM ESTSEMIACAB
             WHERE (CDFILIAL  = @V_SEMIACABADO_CDFILIAL)
               AND (NRPEDIDO  = @P_NRPEDIDO  )
               AND (CDPRODUTO = @V_CDPRODESTO);
              
            IF (NOT @V_CDPRODEMANDA IS NULL) BEGIN
            
               -- Altera a quantidade da demanda deste semiacabado.
               UPDATE ESTSEMIACAB
                  SET QTDEMANDA  = @V_QTDEMANDA + @V_SEMIACABADO_QTAINSUMPREOP
                WHERE (CDFILIAL  = @V_SEMIACABADO_CDFILIAL)
                  AND (NRPEDIDO  = @P_NRPEDIDO    )
                  AND (CDPRODUTO = @V_CDPRODEMANDA);
            END
            ELSE BEGIN
               -- Insere a demanda e a quantidade disponvel do semiacabado.
               INSERT INTO ESTSEMIACAB (CDFILIAL, NRPEDIDO, CDPRODUTO, QTDEMANDA, QTESTOQUE)
               VALUES (@V_SEMIACABADO_CDFILIAL, @P_NRPEDIDO, @V_CDPRODESTO, @V_SEMIACABADO_QTAINSUMPREOP, @V_QTESTOQDIA);
               
            END;
            
            SET @V_NOVAQT = @V_SEMIACABADO_QTAINSUMPREOP - @V_QTESTOQDIA;

         END;
      END;
   
      IF @V_NOVAQT > 0 BEGIN
      
         
         EXECUTE ESTOQUE_SEMIACABADO @V_SEMIACABADO_CDFILIAL, @P_NRPEDIDO, @V_SEMIACABADO_CDPRODINSUMO, @V_SEMIACABADO_CDPROCFABRIC, @V_NOVAQT;

      END;
      
      FETCH NEXT FROM C_SEMIACABADO INTO @V_SEMIACABADO_CDFILIAL, @V_SEMIACABADO_CDPRODUTO, @V_SEMIACABADO_CDPRODINSUMO,
                                         @V_SEMIACABADO_CDPROCFABRIC, @V_SEMIACABADO_QTAINSUMPREOP;
   END;
   CLOSE C_SEMIACABADO;
   DEALLOCATE C_SEMIACABADO;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_OP_PREVCOMP','6.05.xxx';
GO

